==> techne-0.1.tar.gz: techne-0.1/src/angular.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "angular.h"

@implementation Angular

-(Joint *) init
{
    [super init];
    
    self->joint = dJointCreateAMotor (_WORLD, NULL);

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;

    self->axes[2][0] = 0;
    self->axes[2][1] = 0;
    self->axes[2][2] = 1;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    self->stops[2][0] = -dInfinity;
    self->stops[2][1] = dInfinity;

    return self;
}

-(void) get
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -1);

    if (!xstrcmp(k, "axes")) {
	dJointGetAMotorAxis ([self joint], 0, self->axes[0]);
	dJointGetAMotorAxis ([self joint], 1, self->axes[1]);
	dJointGetAMotorAxis ([self joint], 2, self->axes[2]);

	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber (_L, self->axes[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "motor")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->motor[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "stops")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);

	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->stops[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 1);
	
	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->hardness[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 2);

	    lua_pushnumber (_L, bounce[j]);
	    lua_rawseti (_L, -2, 3);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "tolerance")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_pushnumber (_L, self->tolerance[j]);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "state")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < self->degrees ; j += 1) {
	    lua_pushnumber (_L, dJointGetAMotorAngle ([self joint], j));
	    lua_rawseti (_L, -2, j + 1);
	    
	    lua_pushnumber (_L, 0/* dJointGetAMotorAngleRate ([self joint], j) */);
	    lua_rawseti (_L, -2, j + self->degrees + 1);
	}
    } else {
	[super get];
    }
}

-(void) set
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -2);

    if (!xstrcmp(k, "axes")) {
        if(lua_istable (_L, 3)) {
	    self->degrees = lua_objlen (_L, 3);
	    
	    for(j = 0 ; j < self->degrees ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->axes[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dSafeNormalize3 (self->axes[j]);
		    dJointSetAMotorAxis([self joint], j,
					dJointGetBody([self joint], 0) ? 1 : 0,
					self->axes[j][0],
					self->axes[j][1],
					self->axes[j][2]);
		}

		lua_pop (_L, 1);
	    }

	    dJointSetAMotorNumAxes ([self joint], self->degrees);
	}
    } else if (!xstrcmp(k, "motor")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->motor[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dJointSetAMotorParam ([self joint],
					  dParamVel + dParamGroup * j,
					  self->motor[j][0]);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamFMax + dParamGroup * j,
					  self->motor[j][1]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "stops")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    lua_rawgeti (_L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber (_L, -1);
		
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);

		    lua_rawgeti (_L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber (_L, -1);
			
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);
	
		    lua_rawgeti (_L, -1, 3);
		    bounce[j] = lua_tonumber (_L, -1);
		    lua_pop (_L, 1);


		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetAMotorParam ([self joint],
					  dParamLoStop + dParamGroup * j,
					  -dInfinity);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamHiStop + dParamGroup * j,
					  dInfinity);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamLoStop + dParamGroup * j,
					  self->stops[j][0]);

		    dJointSetAMotorParam ([self joint],
					  dParamHiStop + dParamGroup * j,
					  self->stops[j][1]);

		    dJointSetAMotorParam ([self joint],
					  dParamStopCFM + dParamGroup * j,
					  self->hardness[j][0]);

		    dJointSetAMotorParam ([self joint],
					  dParamStopERP + dParamGroup * j,
					  self->hardness[j][1]);

		    dJointSetAMotorParam ([self joint],
					  dParamBounce + dParamGroup * j,
					  bounce[j]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "tolerance")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->tolerance[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);

		dJointSetAMotorParam ([self joint],
				      dParamCFM + dParamGroup * j,
				      self->tolerance[j]);
            }
        }
    } else if (!xstrcmp(k, "bodies")) {
	[super set];

	for (i = 0 ; i < self->degrees ; i += 1) {
	    dJointSetAMotorAxis ([self joint], i,
				 dJointGetBody ([self joint], 0) ? 1 : 0,
				 self->axes[i][0],
				 self->axes[i][1],
				 self->axes[i][2]);
	}
    } else {
	[super set];
    }
}

@end

==> techne-0.2.1.tar.gz: techne-0.2.1/src/angular.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "angular.h"

@implementation Angular

-(Joint *) init
{
    char *list[] = {
	"axes", "bodies", "motor", "state", "stops", "tolerance"
    };

    self->joint = dJointCreateAMotor (_WORLD, NULL);

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;

    self->axes[2][0] = 0;
    self->axes[2][1] = 0;
    self->axes[2][2] = 1;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    self->stops[2][0] = -dInfinity;
    self->stops[2][1] = dInfinity;

    [super init];
    [self add: sizeof (list) / sizeof (char *) Properties: list];

    return self;
}

-(void) get
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -1);

    if (!xstrcmp(k, "axes")) {
	dJointGetAMotorAxis ([self joint], 0, self->axes[0]);
	dJointGetAMotorAxis ([self joint], 1, self->axes[1]);
	dJointGetAMotorAxis ([self joint], 2, self->axes[2]);

	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber (_L, self->axes[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "motor")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->motor[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "stops")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);

	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->stops[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 1);
	
	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->hardness[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 2);

	    lua_pushnumber (_L, bounce[j]);
	    lua_rawseti (_L, -2, 3);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "tolerance")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_pushnumber (_L, self->tolerance[j]);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "state")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < self->degrees ; j += 1) {
	    lua_pushnumber (_L, dJointGetAMotorAngle ([self joint], j));
	    lua_rawseti (_L, -2, j + 1);
	    
	    lua_pushnumber (_L, 0/* dJointGetAMotorAngleRate ([self joint], j) */);
	    lua_rawseti (_L, -2, j + self->degrees + 1);
	}
    } else {
	[super get];
    }
}

-(void) set
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -2);

    if (!xstrcmp(k, "axes")) {
        if(lua_istable (_L, 3)) {
	    self->degrees = luaX_objlen (_L, 3);

	    dJointSetAMotorNumAxes ([self joint], self->degrees);
	    
	    for(j = 0 ; j < self->degrees ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->axes[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dSafeNormalize3 (self->axes[j]);
		    dJointSetAMotorAxis([self joint], j,
					dJointGetBody([self joint], 0) ? 1 : 0,
					self->axes[j][0],
					self->axes[j][1],
					self->axes[j][2]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "motor")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->motor[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dJointSetAMotorParam ([self joint],
					  dParamVel + dParamGroup * j,
					  self->motor[j][0]);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamFMax + dParamGroup * j,
					  self->motor[j][1]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "stops")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    lua_rawgeti (_L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber (_L, -1);
		
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);

		    lua_rawgeti (_L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber (_L, -1);
			
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);
	
		    lua_rawgeti (_L, -1, 3);
		    bounce[j] = lua_tonumber (_L, -1);
		    lua_pop (_L, 1);


		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetAMotorParam ([self joint],
					  dParamLoStop + dParamGroup * j,
					  -dInfinity);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamHiStop + dParamGroup * j,
					  dInfinity);
		    
		    dJointSetAMotorParam ([self joint],
					  dParamLoStop + dParamGroup * j,
					  self->stops[j][0]);

		    dJointSetAMotorParam ([self joint],
					  dParamHiStop + dParamGroup * j,
					  self->stops[j][1]);

		    dJointSetAMotorParam ([self joint],
					  dParamStopCFM + dParamGroup * j,
					  self->hardness[j][0]);

		    dJointSetAMotorParam ([self joint],
					  dParamStopERP + dParamGroup * j,
					  self->hardness[j][1]);

		    dJointSetAMotorParam ([self joint],
					  dParamBounce + dParamGroup * j,
					  bounce[j]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "tolerance")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->tolerance[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);

		dJointSetAMotorParam ([self joint],
				      dParamCFM + dParamGroup * j,
				      self->tolerance[j]);
            }
        }
    } else if (!xstrcmp(k, "bodies")) {
	[super set];

	for (i = 0 ; i < self->degrees ; i += 1) {
	    dJointSetAMotorAxis ([self joint], i,
				 dJointGetBody ([self joint], 0) ? 1 : 0,
				 self->axes[i][0],
				 self->axes[i][1],
				 self->axes[i][2]);
	}
    } else {
	[super set];
    }
}

@end
